pub mod bytes;
pub mod content_disposition;
pub mod debug;
pub mod defer;
pub mod hash;
pub mod html;
pub mod json;
pub mod math;
pub mod mutex_map;
pub mod rand;
pub mod set;
pub mod stream;
pub mod string;
pub mod sys;
mod tests;
pub mod time;

pub use ::conduit_macros::implement;
pub use ::ctor::{ctor, dtor};

pub use self::{
	bytes::{increment, u64_from_bytes, u64_from_u8, u64_from_u8x8},
	debug::slice_truncated as debug_slice_truncated,
	hash::calculate_hash,
	html::Escape as HtmlEscape,
	json::{deserialize_from_str, to_canonical_object},
	math::clamp,
	mutex_map::{Guard as MutexMapGuard, MutexMap},
	rand::string as random_string,
	stream::{IterStream, ReadyExt, TryReadyExt},
	string::{str_from_bytes, string_from_bytes},
	sys::available_parallelism,
	time::now_millis as millis_since_unix_epoch,
};

#[inline]
pub fn exchange<T>(state: &mut T, source: T) -> T { std::mem::replace(state, source) }

#[must_use]
pub fn generate_keypair() -> Vec<u8> {
	let mut value = rand::string(8).as_bytes().to_vec();
	value.push(0xFF);
	value.extend_from_slice(
		&ruma::signatures::Ed25519KeyPair::generate().expect("Ed25519KeyPair generation always works (?)"),
	);
	value
}
